==> jijzept-0.13.3-cp39-cp39-macosx_11_0_arm64.whl: jijzept/sampler/base_sampler.py <==
from __future__ import annotations

import dataclasses
import json
import time
import typing as typ

import numpy as np
import jijmodeling as jm
from jijmodeling import Problem

from jijzept.client import JijZeptClient
from jijzept.config.path_type import PATH_TYPE
from jijzept.entity.schema import SolverType
from jijzept.post_api import post_instance_and_query
from jijzept.response import JijModelingResponse


# This class provides only a constructor,
# so each sampler should implement its own methods such as .sample_model.
# In addition, each sampler specifies the API to be connected by specifying "jijmodeling_solver_type",
# so be sure to implement it.
class JijZeptBaseSampler:
    """Parent class for each sampler in JijZept.

    This class only provide the constructor.
    The actual sampling method are implemented by each sampler.
    """

    jijmodeling_solver_type: SolverType

    def __init__(
        self,
        token: str | None = None,
        url: str | None = None,
        proxy: str | None = None,
        config: PATH_TYPE | None = None,
        config_env: str = "default",
    ):
        """Sets token and url.

        If you do not set any arguments, JijZept configuration file is used.
        If you set the url or token here, that will be used as the priority setting for connecting to the API.
        See JijZeptClient for details.

        Args:
            token (str | None, optional): Token string. Defaults to None.
            url (str | None, optional): API URL. Defaults to None.
            proxy (str | None, optional): Proxy URL. Defaults to None.
            config (str | None, optional): Config file path. Defaults to None.
            config_env (str, optional): configure environment name. Defaults to 'default'.

        Raises:
            TypeError: `token`, `url`, or `config` is not str.
        """
        self.client = JijZeptClient(
            url=url, token=token, proxy=proxy, config=config, config_env=config_env
        )


@dataclasses.dataclass
class ParameterSearchParameters:
    multipliers: dict[str, float] = dataclasses.field(default_factory=lambda: {})
    mul_search: bool = False
    num_search: int = 15
    algorithm: str | None = None
    normalize_qubo: bool = True


def sample_model(
    client: JijZeptClient,
    solver: str,
    queue_name: str,
    problem: Problem,
    instance_data: dict[str, list | np.ndarray],
    fixed_variables: dict[str, dict[tuple[int, ...], int]],
    parameter_search_parameters: ParameterSearchParameters,
    timeout: float | None,
    sync: bool,
    **kwargs,
) -> JijModelingResponse:
    start = time.time()

    m_seri = jm.to_serializable(problem)

    _instance_data = {
        k: v.tolist() if isinstance(v, np.ndarray) else v
        for k, v in instance_data.items()
    }

    instance = {
        "mathematical_model": json.dumps(m_seri),
        "instance_data": _instance_data,
    }

    param_dict = dataclasses.asdict(parameter_search_parameters)
    param_dict["fixed_variables"] = jm.utils.serialize_fixed_var(fixed_variables)
    param_dict.update(kwargs)

    response = post_instance_and_query(
        JijModelingResponse,
        client=client,
        instance_type="JijModeling",
        instance=instance,
        queue_name=queue_name,
        solver=solver,
        parameters=param_dict,
        timeout=timeout,
        sync=sync,
    )
    response.measuring_time.total = time.time() - start

    return response


def merge_params_and_kwargs(
    parameters: typ.Type | None, kwargs: dict[str, typ.Any], param_cls: typ.Type
) -> dict[str, typ.Any]:
    from jijzept.sampler import thirdparty

    param_dict: dict[str, typ.Any] = {}
    if parameters is None:
        parameters = dataclasses.asdict(param_cls())
    else:
        parameters = dataclasses.asdict(parameters)

    if param_cls.__name__ in thirdparty.__all__:
        param_dict.update({"parameters": parameters})
    else:
        param_dict.update(parameters)
    
    for key, value in kwargs.items():
        # The value put in the Parameter class takes precedence.
        param_dict[key] = value
    return param_dict

==> jijzept-1.14.0-cp39-cp39-macosx_12_0_arm64.whl: jijzept/sampler/base_sampler.py <==
from __future__ import annotations

import dataclasses
import time
import typing as typ

import rapidjson

import jijmodeling as jm
import numpy as np

from jijmodeling import Problem

from jijzept.client import JijZeptClient
from jijzept.config.path_type import PATH_TYPE
from jijzept.entity.schema import SolverType
from jijzept.instance_translator.instance_translator import InstanceTranslator
from jijzept.post_api import post_instance_and_query
from jijzept.response import JijModelingResponse
from jijzept.response.base import APIStatus
from jijzept.exception.exception import JijZeptSolvingFailedError, JijZeptSolvingUnknownError, JijZeptSolvingValidationError

# This class provides only a constructor,
# so each sampler should implement its own methods such as .sample_model.
# In addition, each sampler specifies the API to be connected by specifying "jijmodeling_solver_type",
# so be sure to implement it.
class JijZeptBaseSampler:
    """Parent class for each sampler in JijZept.

    This class only provide the constructor.
    The actual sampling method are implemented by each sampler.
    """

    jijmodeling_solver_type: SolverType

    def __init__(
        self,
        token: str | None = None,
        url: str | None = None,
        proxy: str | None = None,
        config: PATH_TYPE | None = None,
        config_env: str = "default",
    ):
        """Sets token and url.

        If you do not set any arguments, JijZept configuration file is used.
        If you set the url or token here, that will be used as the priority setting for connecting to the API.
        See JijZeptClient for details.

        Args:
            token (str | None, optional): Token string. Defaults to None.
            url (str | None, optional): API URL. Defaults to None.
            proxy (str | None, optional): Proxy URL. Defaults to None.
            config (str | None, optional): Config file path. Defaults to None.
            config_env (str, optional): configure environment name. Defaults to 'default'.

        Raises:
            TypeError: `token`, `url`, or `config` is not str.
        """
        self.client = JijZeptClient(
            url=url, token=token, proxy=proxy, config=config, config_env=config_env
        )


@dataclasses.dataclass
class ParameterSearchParameters:
    multipliers: dict[str, float] = dataclasses.field(default_factory=lambda: {})
    mul_search: bool = False
    num_search: int = 15
    algorithm: str | None = None
    normalize_qubo: bool = True


def sample_model(
    client: JijZeptClient,
    solver: str,
    queue_name: str,
    problem: Problem,
    instance_data: dict[str, list | np.ndarray],
    fixed_variables: dict[str, dict[tuple[int, ...], int]],
    parameter_search_parameters: ParameterSearchParameters,
    timeout: float | None,
    sync: bool,
    **kwargs,
) -> JijModelingResponse:
    start = time.time()
    
    mathematical_model = rapidjson.dumps(
        jm.to_serializable(problem),
        ensure_ascii=False,
        write_mode=rapidjson.WM_COMPACT,
    )

    instance = {
        "mathematical_model": mathematical_model,
        "instance_data": InstanceTranslator.instance_translate(instance_data),
    }

    param_dict = dataclasses.asdict(parameter_search_parameters)
    param_dict["fixed_variables"] = jm.utils.serialize_fixed_var(fixed_variables)
    param_dict.update(kwargs)

    response = post_instance_and_query(
        JijModelingResponse,
        client=client,
        instance_type="JijModeling",
        instance=instance,
        queue_name=queue_name,
        solver=solver,
        parameters=param_dict,
        timeout=timeout,
        sync=sync,
    )
    response.measuring_time.total = time.time() - start

    # Raise error if the problem is not solved.
    if response.status == APIStatus.FAILED:
        raise JijZeptSolvingFailedError(
            response.error_message.get('message', "The problem is not solved.")
        )
    elif response.status == APIStatus.UNKNOWNERROR:
        raise JijZeptSolvingUnknownError(
            response.error_message.get('message', "The problem is not solved.")
        )
    elif response.status == APIStatus.VALIDATIONERROR:
        raise JijZeptSolvingValidationError(
            response.error_message.get('message', "The problem is not solved.")
        )

    return response


def merge_params_and_kwargs(
    parameters: typ.Type | None, kwargs: dict[str, typ.Any], param_cls: typ.Type
) -> dict[str, typ.Any]:
    from jijzept.sampler import thirdparty

    param_dict: dict[str, typ.Any] = {}
    if parameters is None:
        parameters = dataclasses.asdict(param_cls())
    else:
        parameters = dataclasses.asdict(parameters)

    if param_cls.__name__ in thirdparty.__all__:
        param_dict.update({"parameters": parameters})
    else:
        param_dict.update(parameters)

    for key, value in kwargs.items():
        # The value put in the Parameter class takes precedence.
        param_dict[key] = value
    return param_dict
